// weatherServer.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"

#include <stdio.h>
#include <string>
using namespace std;

#include <json.h>
#include <http_listener.h>
#include <http_client.h>
#include <uri.h>
using namespace web;
using namespace http;
using namespace http::client;
using namespace utility;
using namespace http::experimental::listener;

const wstring HtmlFormat = L"<!DOCTYPE html>\r\n\
<html lang=\"en\" xmlns = \"http://www.w3.org/1999/xhtml\">\r\n\
<head>\r\n\
<meta charset = \"utf-8\"/>\r\n\
<title></title>\r\n\
    <style type = \"text/css\">\r\n\
        body{\r\n\
            font-family: \"Segoe UI\"; font-size: 24px;\r\n\
            color: #f0e2a6; background-color: #496bb2;\r\n\
        }\r\n\
        table, tr, td, th, tbody, thead{ vertical-align:central }\r\n\
    </style>\r\n\
</head>\r\n\
<body>\r\n\
    <table cellpadding=\"0\" cellspacing=\"0\">\r\n\
        <tr>\r\n\
            <td style = \"color: white; font-size: 32px; font-weight: 400\">%s (%s)</td>\r\n\
            <td rowspan = \"4\">\r\n\
                <img align = \"right\" src = \"http://www.weather-24.com/static/img/weather/9/%s.png\"/>\r\n\
            </td>\r\n\
        </tr>\r\n\
        <tr>\r\n\
            <td>%s</td>\r\n\
        </tr>\r\n\
        <tr>\r\n\
            <td>%.1f (min %.0f, max %.0f)</td>\r\n\
        </tr>\r\n\
        <tr>\r\n\
            <td>%s</td>\r\n\
        </tr>\r\n\
        <tr style = \"height: 400px;\">\r\n\
            <td colspan = \"2\">\r\n\
                <img width = \"480\" height = \"360\"\r\n\
                    src = \"http://dev.virtualearth.net/REST/v1/Imagery/Map/Aerial/%.2f,%.2f/4?mapSize=600,400&pushPin=%.2f,%.2f;52&key=Ap8tYSHbW2282njkPdA1pqOF5MMXiwpQvw1gVy1X4nrGjqV1E2lCy1hafZs1Dbwm\"/>\r\n\
            </td>\r\n\
        </tr>\r\n\
    </table>\r\n\
</body>\r\n\
</html>";

struct WeatherData
{
    WeatherData(
        const wstring& city,
        const wstring& nation,
        datetime dt,
        double currTemp,
        double minTemp,
        double maxTemp,
        const wstring& description,
        const wstring& icon,
		double longitude,
		double latitude) :
        _city(city),
        _nation(nation),
        _dateTime(dt),
        _currTemp(currTemp),
        _minTemp(minTemp),
        _maxTemp(maxTemp),
        _description(description),
        _icon(icon),
		_longitude(longitude),
		_latitude(latitude)
    {
    }
    wstring _city;
    wstring _nation;
    datetime _dateTime;
    double _currTemp;
    double _minTemp;
    double _maxTemp;
    wstring _description;
    wstring _icon;
	double _longitude;
	double _latitude;
};

struct WeatherClient
{
    static pplx::task<shared_ptr<WeatherData>> get(const wstring& city)
    {
        http_client client(L"http://api.openweathermap.org/");
        
		auto uri = L"/data/2.5/weather?q=" + city + L"&units=metric";
        auto task = client.request(methods::GET, uri)
            .then([](http_response response) -> pplx::task<json::value>
        {
            if (response.status_code() == status_codes::OK) {
                return response.extract_json();
            }
            else {
                return pplx::task_from_result(json::value());
            }
        }).then([](json::value value)
        {
            return std::make_shared<WeatherData>(
                value[L"name"].as_string(),
                value[L"sys"][L"country"].as_string(),
                datetime::utc_now(),
                value[L"main"][L"temp"].as_double(),
                value[L"main"][L"temp_min"].as_double(),
                value[L"main"][L"temp_max"].as_double(),
                value[L"weather"][0][L"description"].as_string(),
                value[L"weather"][0][L"icon"].as_string(),
				value[L"coord"][L"lon"].as_double(),
				value[L"coord"][L"lat"].as_double()
            );
        });

        return task;
    }
};

class WeatherServer
{
public:
    WeatherServer(const wstring& url) : _listener(url)
    {
        _listener.support(methods::GET,
            std::bind(&WeatherServer::handle_get, this, std::placeholders::_1));
    }

    pplx::task<void> open() { return _listener.open(); }
    pplx::task<void> close() { return _listener.close(); }

private:
    void handle_get(http_request message)
    {
        auto params = uri::split_query(message.relative_uri().query());
        auto city = params[L"city"];
        WeatherClient::get(city).then([message](std::shared_ptr<WeatherData> data) {
            wchar_t buff[4096];
            swprintf(buff, 4096, HtmlFormat.c_str(),
                data->_city.c_str(),
                data->_nation.c_str(),
                data->_icon.c_str(),
                data->_dateTime.to_string().c_str(),
                data->_currTemp,
                data->_minTemp,
                data->_maxTemp,
                data->_description.c_str(),
				data->_latitude, data->_longitude,
				data->_latitude, data->_longitude);

            message.reply(status_codes::OK, buff, L"text/html");
        });
    }

    http_listener _listener;
};
std::unique_ptr<WeatherServer> s_weatherServer;

void on_initialize(const wstring& address)
{
	uri_builder uri(address);
    uri.append_path(U("weather"));

    auto addr = uri.to_uri().to_string();
    s_weatherServer = std::unique_ptr<WeatherServer>(new WeatherServer(addr));
    s_weatherServer->open().wait();

    ucout << utility::string_t(U("Listening for requests at: ")) << addr << std::endl;

    return;
}

void on_shutdown()
{
    s_weatherServer->close().wait();
}

int _tmain(int argc, _TCHAR* argv[])
{
    utility::string_t address = U("http://localhost:8080");

    on_initialize(address);
    std::cout << "Press ENTER to exit." << std::endl;

    std::string line;
    std::getline(std::cin, line);

    on_shutdown();
    return 0;
}


//Data data;
//message.reply(status_codes::OK, data.AsJSON());
//#define ID U("Id")
//#define LEN U("Len")
//class Data
//{
//public:
//	web::json::value AsJSON() const
//	{
//		web::json::value result = web::json::value::object();
//		result[ID] = web::json::value::number((double)42.0);
//		result[LEN] = web::json::value::number(11);
//		return result;
//	}
//};

